"""
@ Qufilab, 2020.
@ Anton Normelius

Python interface for volume indicators.

"""
import time
import numpy as np 

from qufilab.indicators._volume import *

def acdi(close, high, low, volume):
    """
    .. Accumulation distribution

    Parameters
    ----------
    close : `ndarray`
        Array of type float64 or float32 containing closing prices.
    high : `ndarray`
        Array of type float64 or float32 containing high prices.
    low : `ndarray`
        Array of type float64 or float32 containing low prices.
    volume : `ndarray`
        Array of type float64 or float32 containing volume values.

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the calculated
        accumulation distribution values.
    """
    return acdi_calc(close, high, low, volume)

def obv(price, volume):
    """
    .. On balance volume

    Parameters
    ----------
    price : `ndarray`
        Array of type float64 or float32 containing prices.
    volume : `ndarray`
        Array of type float64 or float32 containing volume values.

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the calculated
        on balance volume values.
    """

    return obv_calc(price, volume)

def cmf(close, high, low, volume, period = 21):
    """
    .. Chaikin money flow

    Parameters
    ----------
    close : `ndarray`
        Array of type float64 or float32 containing closing prices.
    high : `ndarray`
        Array of type float64 or float32 containing high prices.
    low : `ndarray`
        Array of type float64 or float32 containing low prices.
    volume : `ndarray`
        Array of type float64 or float32 containing volume values.
    period : `int`, optional
        Number of periods to use.
        Defaults to 21.

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the calculated
        chaikin money flow values.
    """
    return cmf_calc(close, high, low, volume, period)

def ci(close, high, low, volume):
    """
    .. Chaikin indicator

    Parameters
    ----------
    close : `ndarray`
        Array of type float64 or float32 containing closing prices.
    high : `ndarray`
        Array of type float64 or float32 containing high prices.
    low : `ndarray`
        Array of type float64 or float32 containing low prices.
    volume : `ndarray`
        Array of type float64 or float32 containing volume values.

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the calculated
        chaikin indicator values.
    """
    return ci_calc(close, high, low, volume)

def pvi(price, volume):
    """
    .. Positive volume index

    Parameters
    ----------
    price : `ndarray`
        Array of type float64 or float32 containing prices.
    volume : `ndarray`
        Array of type float64 or float32 containing volume values.

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the calculated
        positive volume index values.
    """
    return pvi_calc(price, volume)

def nvi(price, volume):
    """
    .. Negative volume index

    Parameters
    ----------
    price : `ndarray`
        Array of type float64 or float32 containing prices.
    volume : `ndarray`
        Array of type float64 or float32 containing volume values.

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the calculated
        negative volume index values.
    """
    return nvi_calc(price, volume)



"""
@ Qufilab, 2020.
@ Anton Normelius

Python interface for available indicators.

"""
import time
import numpy as np 

from qufilab.indicators._trend import *
from qufilab.indicators._volatility import *
from qufilab.indicators._momentum import *
from qufilab.indicators._volume import *
from qufilab.indicators._stat import *

def _validate_input(data = None, period = None):
    """
    General function to validate specified user parameters.
    """
    if data is not None:
        if not isinstance(data, np.ndarray) or data.dtype != np.float64:
            raise ValueError("Param 'data' needs to be of type numpy.ndarray with " \
                "dtype numpy.float64, i.e. double")
    
    if period is not None:
        if not isinstance(period, int) or period <= 0:
            raise ValueError("Param 'period' needs to be an int greater than zero")


# Trend interface
# ---------------
#def sma(data, period):
#    """
#    Simple moving average
#
#
#    ----------
#    data : `ndarray`
#        Array of type float64 containing the data to be used.
#    period : `int`
#        Number of periods to be used.
#    
#    Returns
#    -------
#    `ndarray`
#        Returns a numpy ndarray of type float64 with simple moving averages.
#    
#    Examples
#    --------
#    >>> import qufilab as ql
#    >>> data = np.array([1.0, 2.0, 3.0, 4.0, 5.0])
#    >>> sma = ql.sma(data, 2)
#    [nan 1.5 2.5 3.5 4.5]
#
#    Notes
#    -----
#    The calculation of sma is a equally weighted mean for the last *n* days.
#
#    .. math:: 
#        sma_K = \\frac{price_K + price_{K-1} + ... + price_{K-(n-1)}}{n} = 
#        \\frac{1}{n}\sum_{i=0}^{n-1} price_{K-i}
#        
#    """
#    #_validate_input(data = data, period = period)
#    return sma_calc(data, period)
#
#def sma_test(data, period):
#    return sma_calc_test(data, period)
#
#def ema(data, period):
#    """
#    Exponential moving average
#    
#    Parameters
#    ----------
#    data : `ndarray`
#        Array of type float64 containing the data to be used.
#    period : `int`
#        Number of periods to be used.
#    
#    Returns
#    -------
#    `ndarray`
#        Returns a ndarray of type float64 with exponential moving averages.
#
#    Examples
#    --------
#    >>> import qufilab as ql
#    >>> data = np.array([1.0, 2.0, 3.0, 4.0, 5.0])
#    >>> ema = ql.ema(data, 2)
#    [nan 1.5 2.5 3.5 4.5]
#
#    Notes
#    -----
#    The calculation of ema can be written as a power series:
#
#    .. math:: ema_K = \\alpha[price_K + (1-\\alpha)price_{K-1} + \\
#        (1-\\alpha)^2price_{K-2}  + ... +  (1-\\alpha)^{K-(n-1)}price_{K-(n-1)},
#    
#    which also can be written as:
#
#    .. math:: 
#        ema_K = ema_{K-1} + \\alpha[price_K - ema_{K-1}], 
#
#    where :math:`\\alpha = \\frac{2}{n+1}` is the multiplier and depends on 
#    the period *n*. Observe that for the first ema value, a simple moving average
#    is used.
#    """
#    _validate_input(data = data, period = period)
#    return ema_calc(data, period)
#
#
#def dema(data, period):
#    """
#    Double exponential moving average
#    
#    Parameters
#    ----------
#    data : `ndarray`
#        Array of type float64 containing the data to be used.
#    period : `int`
#        Number of periods to be used.
#    
#    Returns
#    -------
#    `ndarray`
#        Returns a ndarray of type float64 with double exponential moving averages.
#
#    Examples
#    --------
#    >>> import qufilab as ql
#    >>> data = np.array([1.0, 2.0, 3.0, 4.0, 5.0])
#    >>> dema = ql.dema(data, 2)
#    [nan nan 3. 4. 5.]
#
#    Notes
#    -----
#    The calculation of dema depends on taking an ema of an ema, hence the name.
#    Since an ema is calculated from an ema, one more value is needed in the 
#    original series to start calculating values, which is shown in the example above.
#
#    .. math:: 
#        dema_K = 2 \cdot ema_K - ema(ema_K)
#
#    """
#    _validate_input(data = data, period = period)
#    return dema_calc(data, period)
#
#def tema(data, period):
#    """
#    Triple exponential moving average
#
#    Parameters
#    ----------
#    data : `ndarray`
#        Array of type float64 containing the data to be used.
#    period : `int`
#        Number of periods to be used.
#    
#    Returns
#    -------
#    `ndarray`
#        Returns a ndarray of type float64 with triple exponential moving averages.
#
#    Examples
#    --------
#    >>> import qufilab as ql
#    >>> data = np.array([1.0, 2.0, 3.0, 4.0, 5.0])
#    >>> tema = ql.tema(data, 3)
#    [nan nan nan 4. 5.]
#
#    Notes
#    -----
#    The triple exponential moving average is similar to the other exponential
#    moving averages, however this one is utilizing three exponential moving averages.
#
#    .. math:: 
#        tema_K = 3 \cdot ema_K - 3 \cdot ema(ema_K) + ema(ema(ema_K))
#    """
#    _validate_input(data = data, period = period)
#    return tema_calc(data, period)
#
#def t3(data, period, volume_factor = 0.7):
#    """
#    T3 moving average
#
#    Parameters
#    ----------
#    data : `ndarray`
#        Array of type float64 containing the data to be used.
#    period : `int`
#        Number of periods to be used.
#    volume_factor : `float`, optional
#        Volume factor to be used when calculating the constants.
#    
#    Returns
#    -------
#    `ndarray`
#        Returns a ndarray of type float64 with t3 moving averages.
#
#    Examples
#    --------
#    >>> import qufilab as ql
#    >>> data = np.array([1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0])
#    >>> t3 = ql.t3(data, 2)
#    [nan  nan  nan  nan  nan  nan  nan  nan  nan 9.55]
#
#    Notes
#    -----
#    The t3 moving average indicator utilizes many different forms of the 
#    exponential moving average and is calculated with:
#
#    .. math:: 
#        t3 = c_1\cdot e_6 + c_2\cdot e_5 + c_3\cdot e_4 + c_4\cdot e_3 
#
#        ema_1 = ema(price) 
#
#        ema_2 = ema(ema_1) 
#
#        ema_3 = ema(ema_2) 
#
#        ema_4 = ema(ema_3) 
#
#        ema_5 = ema(ema_4) 
#
#        ema_6 = ema(ema_5)
#
#        c_1 = -a^3
#
#        c_2 = 3a^2 + 3a^3 
#
#        c_3 = –6a^2–3a–3a^3
#
#        c_4 = 1 + 3a + a^3 + 3a^2
#
#    where *a* is the volume factor and is typically set to :math:`0.7`.
#
#    """
#    _validate_input(data = data, period = period)
#
#    if volume_factor < 0:
#        raise ValueError("Param 'volume_factor' needs to be bigger than zero")
#
#    return t3_calc(data, period, volume_factor)
#
#
#def tma(data, period):
#    """
#    Triangular moving average
#
#    Parameters
#    ----------
#    data : `ndarray`
#        Array of type float64 containing the data to be used.
#    period : `int`
#        Number of periods to be used.
#    
#    Returns
#    -------
#    `ndarray`
#        Returns a ndarray of type float64 with triangular moving averages.
#
#    Examples
#    --------
#    >>> import qufilab as ql
#    >>> data = np.array([1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0])
#    >>> tma = ql.tma(data, 2)
#    [nan 1.5 2.5 3.5 4.5 5.5 6.5 7.5 8.5 9.5]
#
#    Notes
#    -----
#    The triangular moving average is calculated by taking an sma of an sma.
#
#    .. math::
#        tma = sma(sma(price, n_1), n_2)
#
#    As seen in the formula above, two different periods are used in this 
#    implementation. If the user specified period is even:
#
#    .. math::
#        n_1 = \\frac{period}{2}
#
#        n_2 = \\frac{period}{2} + 1
#    
#    If the user specified period is uneven:
#
#    .. math::
#        n_1 = n_2 = \\frac{period + 1}{2} (\\textrm{rounded up})
#    """
#    _validate_input(data = data, period = period)
#
#    return tma_calc(data, period)
#
#def smma(data, period):
#    """
#    Smoothed moving average
#    
#    Parameters
#    ----------
#    data : `ndarray`
#        Array of type float64 containing the data to be used.
#    period : `int`
#        Number of periods to be used.
#    
#    Returns
#    -------
#    `ndarray`
#        Returns a ndarray of type float64 with triangular moving averages.
#    """
#    _validate_input(data = data, period = period)
#    return smma_calc(data, period)
#
#def lwma(data, period):
#    """
#    Linear weighted moving average
#
#    Parameters
#    ----------
#    data : `ndarray`
#        Array of type float64 containing the data to be used.
#    period : `int`
#        Number of periods to be used.
#    
#    Returns
#    -------
#    `ndarray`
#        Returns a ndarray of type float64 with linear weighted moving averages.
#
#    Examples
#    --------
#    >>> import qufilab as ql
#    >>> data = np.array([1.0, 2.0, 3.0, 4.0, 5.0])
#    >>> lwma = ql.lwma(data, 2)
#    [nan 1.66666667 2.66666667 3.66666667 4.66666667]
#
#    Notes
#    -----
#    The linear weighted moving average is calculated similar to the simple
#    moving average, except that the prices aren't equally weighted, but
#    linearly weighted with the highest weight going first and then decreasing
#    linearly. This implementation utilizes the number of periods as the highest
#    weight, and thereafter decreasing down to one.
#
#    .. math:: lwma_K = \\frac{price_K \cdot w_n + price_{K-1} \cdot w_{n-1} \\
#            + ... + price_{K-(n-1)} \cdot w_1}{\sum_{i=1}^n w_i},
#
#    where :math:`w_1 = 1, w_2 = 2,... w_n = n`
#    """
#    _validate_input(data = data, period = period)
#    return lwma_calc(data, period)
#
#def wc(close, high, low):
#    """
#    Weighted close
#    
#    Parameters
#    ----------
#    close : `ndarray`
#        Array of type float64 containing the closing prices to be used.
#    high : `ndarray`
#        Array of type float64 containing the high prices to be used.
#    low : `ndarray`
#        Array of type float64 containing the low prices to be used.
#    
#    Returns
#    -------
#    `ndarray`
#        Returns a ndarray of type float64 with weighted close values.
#
#    Examples
#    --------
#    >>> import qufilab as ql
#    >>> close  = np.array([5.0, 15.0, 25.0, 35.0, 45.0])
#    >>> high = np.array([10.0, 20.0, 30.0, 40.0, 50.0])
#    >>> low = np.array([1.0, 11.0, 21.0, 31.0, 41.0])
#    >>> wc = ql.wc(close, high, low)
#    [5.25 15.25 25.25 35.25 45.25]
#
#    Notes
#    -----
#    The weighted close is defined as:
#
#    .. math:: wc_K = \\frac{2 \cdot close_K + high_K + low_K}{4}
#
#    """
#    if not isinstance(close, np.ndarray) or close.dtype != np.float64:
#        raise ValueError("Param 'close' needs to be of type numpy.ndarray with " \
#            "dtype numpy.float64, i.e. double")
#
#    if not isinstance(high, np.ndarray) or high.dtype != np.float64:
#        raise ValueError("Param 'high' needs to be of type numpy.ndarray with " \
#            "dtype numpy.float64, i.e. double")
#
#    if not isinstance(low, np.ndarray) or low.dtype != np.float64:
#        raise ValueError("Param 'low' needs to be of type numpy.ndarray with " \
#            "dtype numpy.float64, i.e. double")
#
#    return wc_calc(close, high, low)

# Volatility interface
# --------------------
#def bbands(data, period, deviation = 2):
#    """
#    Bollinger bands
#    """
#    return bbands_calc(data, period, deviation)
#
#def kc(close, high, low, period = 20, period_atr = 20, deviation = 2):
#    """
#    Keltner channels
#    """
#    return kc_calc(close, high, low, period, period_atr, deviation)
#
#def atr(prices, highs, lows, periods):
#    """
#    Average true range
#    """
#    return atr_calc(prices, highs, lows, periods)
#
#def cv(highs, lows, period = 10, smoothing_period = 10):
#    """ 
#    Chaikin volatility
#    """
#    return cv_calc(highs, lows, period, smoothing_period)
#
## Momentum interface
## -----------------
#def rsi(data, periods, rsi_type = "smoothed"):
#    """
#    Relative strength index
#    """
#    return rsi_calc(data, periods, rsi_type.lower())
#
#def macd(data):
#    """
#    Macd
#    """
#    return macd_calc(data)
#
#def willr(data, highs, lows, periods):
#    """
#    William's R
#    """
#    return willr_calc(data, highs, lows, periods)
#
#def roc(data, period):
#    """
#    Price rate of change
#    """
#    return roc_calc(data, period)
#
#def vpt(data, volumes):
#    """
#    Volume price trend
#    """
#    return vpt_calc(data, volumes)
#
#def mi(data, periods):
#    """
#    Momentum indicator
#    """
#    return mi_calc(data, periods)
#
#def apo(data, period_slow = 26, period_fast = 12, ma = "sma"):
#    """
#    Absolute price oscillator
#    """
#    if ma.lower() not in ["sma", "ema"]:
#        raise ValueError("param 'ma' needs to be 'ema' or 'sma'")
#
#    return apo_calc(data, period_slow, period_fast, ma.lower())
#
#def bop(high, low, open_, close):
#    """
#    Balance of power
#    """
#    return bop_calc(high, low, open_, close)
#
#def cmo(close, period):
#    """
#    Chande momentum indicator
#    """
#    return cmo_calc(close, period)
#
#def mfi(high, low, close, volume, period):
#    """
#    Money flow index
#    """
#    return mfi_calc(high, low, close, volume, period)
#
#def ppo(prices, period_fast = 12, period_slow = 26, ma_type = "ema"):
#    """
#    Percentage price oscillator
#    """
#    if ma_type.lower() not in ["sma", "ema"]:
#        raise ValueError("Param 'ma_type' needs to be 'sma' or 'ema'")
#
#    return ppo_calc(prices, period_fast, period_slow, ma_type)
#
##def stochastic(close, high, low, mode = "fast", period_k = 10, method = "ema"):
##    """
##    Returns %K and %D stochastics.
##
##    :param close: Closing prices.
##    :type close: List.
##    :param high: High prices.
##    :type high: List.
##    :param low: Low prices.
##    :type low: List.
##    :param mode: Specify whether to calculate 'fast' or 'slow' stochastic.
##    :type mode: Str.
##    :param period_k: Specify the number of periods used in the Stochastic (%K)
##        calculation.
##    :type period_k: Int.
##    :param method: Specify the method that is used to calculate Stochastic (%D) 
##        calculation.
##    :type method: Str.
##    :return tuple: A tuple containing (%K, %D).
##    """
##    if method and method.lower() not in ['sma', 'ema']:
##        raise ValueError("Param 'method' needs to be 'sma' or 'ema'.")
##    
##    if mode and mode.lower() not in ['fast', 'slow']:
##        raise ValueError("Param 'mode' needs to be 'fast' or 'slow'.")
##    
##    if not isinstance(period_k, int):
##        raise ValueError("Param 'period_k' needs to be an int.")
##
##    return stochastic_calc(close, high, low, mode, period_k, method)
#
#
#def cci(close, high, low, period = 20):
#    return cci_calc(close, high, low, period)
#
#def aroon(high, low, period = 20):
#    return aroon_calc(high, low, period)
#
##def tsi(close, period = 25, period_double = 13):
##    return tsi_calc(close, period, period_double)
#
#
## Volume interface
## ---------------
#def acdi(prices, highs, lows, volumes):
#    return acdi_calc(prices, highs, lows, volumes)
#
#def obv(prices, volumes):
#    return obv_calc(prices, volumes)
#
#def cmf(prices, highs, lows, volumes, periods = 21):
#    return cmf_calc(prices, highs, lows, volumes, periods)
#
#def ci(prices, highs, lows, volumes):
#    return ci_calc(prices, highs, lows, volumes)
#
#def pvi(prices, volumes):
#    return pvi_calc(prices, volumes)
#
#def nvi(prices, volumes):
#    return nvi_calc(prices, volumes)
#
#
## Stat interface
## -------------
#
#def std(data, period, normalize = True):
#    return std_calc(data, period, normalize)
#
#def var(data, period, normalize = True):
#    return var_calc(data, period, normalize)
#
#def cov(data, market, period, normalize = True):
#    return cov_calc(data, market, period, normalize)
#
#def beta(prices, market, period, normalize = False):
#    return beta_calc(prices, market, period, normalize)
#
#def pct_change(prices, period):
#    return pct_change_calc(prices, period)
#
# SSE TESTING
#def sse(price, high, low):
#    return sse_calc(price, high, low);






"""
@ Qufilab, 2020.
@ Anton Normelius

Python interface for trend indicators.

"""
import time
import numpy as np

from qufilab.indicators._trend import *


def sma(data, periods):
    """
    .. Simple moving average

    Parameters
    ----------
    data : `ndarray`
        An array containing values.
    periods : `int`
        Number of periods to be used.

    Returns
    -------
    `ndarray`
        An array containing calculated simple moving average values.

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ... 
    >>> # Load a sample dataframe.
    >>> df = ql.load_sample('MSFT')
    >>> print(df['close'].dtype)
    float64
    >>> sma = ql.sma(df['close'], periods = 10)
    >>> print(sma)
    [nan nan nan ... 209.872 209.695 209.749]

    Notes
    -----
    The calculation of sma is a equally weighted mean for the last *n* days.

    .. math::
        sma_K = \\frac{price_K + price_{K-1} + ... + price_{K-(n-1)}}{n} =
        \\frac{1}{n}\sum_{i=0}^{n-1} price_{K-i}

    """
    return sma_calc(data, periods)

def ema(data, periods):
    """
    .. Exponential Moving Average

    Parameters
    ----------
    data : `ndarray`
        An array containing values.
    periods : `int`
        Number of periods to be used.

    Returns
    -------
    `ndarray`
        An array containing calculated exponential moving average values.

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ... 
    >>> # Load a sample dataframe.
    >>> df = ql.load_sample('MSFT')
    >>> print(df['close'].dtype)
    float64
    >>> ema = ql.ema(df['close'], periods = 10)
    >>> print(ema)
    [nan nan nan ... 209.63323895 210.53265005 210.98489549]

    Notes
    -----
    The calculation of ema can be written as a power series:

    .. math:: ema_K = \\alpha[price_K + (1-\\alpha)price_{K-1} + \\
        (1-\\alpha)^2price_{K-2}  + ... +  (1-\\alpha)^{K-(n-1)}price_{K-(n-1)},

    which also can be written as:

    .. math::
        ema_K = ema_{K-1} + \\alpha[price_K - ema_{K-1}],

    where :math:`\\alpha = \\frac{2}{n+1}` is the multiplier and depends on
    the period *n*. Observe that for the first ema value, a simple moving average
    is used.
    """
    return ema_calc(data, periods)


def dema(data, periods):
    """
    .. Double Exponential Moving Average

    Parameters
    ----------
    data : `ndarray`
        An array containing values.
    periods : `int`
        Number of periods to be used.

    Returns
    -------
    `ndarray`
        An array containing calculated double exponential moving average values.

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ... 
    >>> # Load a sample dataframe.
    >>> df = ql.load_sample('MSFT')
    >>> print(df['close'].dtype)
    float64
    >>> dema = ql.ema(df['close'], periods = 10)
    >>> print(dema)
    [nan nan nan ... 210.18599259 211.72078484 212.32702478]

    Notes
    -----
    .. math::
        dema_K = 2 \cdot ema_K - ema(ema_K)

    """
    return dema_calc(data, periods)

def tema(data, periods):
    """
    .. Triple Exponential Moving Average

    Parameters
    ----------
    data : `ndarray`
        An array containing values.
    periods : `int`
        Number of periods to be used.

    Returns
    -------
    `ndarray`
        An array containing calculated triple exponential moving average values.

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ... 
    >>> # Load a sample dataframe.
    >>> df = ql.load_sample('MSFT')
    >>> print(df['close'].dtype)
    float64
    >>> tema = ql.tema(df['close'], periods = 10)
    >>> print(tema)
    [nan nan nan ... 210.00373443 212.09152183 212.75635054]

    Notes
    -----
    The triple exponential moving average is similar to the other exponential
    moving averages, however this one utilize three exponential moving averages.

    .. math::
        tema_K = 3 \cdot ema_K - 3 \cdot ema(ema_K) + ema(ema(ema_K))
    """
    return tema_calc(data, periods)

def t3(data, periods, volume_factor = 0.7):
    """
    .. T3 Moving Average

    Parameters
    ----------
    data : `ndarray`
        An array containing values.
    periods : `int`
        Number of periods to be used.
    volume_factor : `float`, optional
        What volume factor to be used when calculating the constants. 
        See `Notes` below for implementation.

    Returns
    -------
    `ndarray`
        An array containing calculated t3 moving average values.

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ... 
    >>> # Load a sample dataframe.
    >>> df = ql.load_sample('MSFT')
    >>> print(df['close'].dtype)
    float64
    >>> t3 = ql.t3(df['close'], periods = 10)
    >>> print(t3)
    [nan nan nan ... 210.08668472 210.2348457 210.47802463]

    Notes
    -----
    The t3 moving average indicator utilize many different 
    exponential moving averages and is calculated with:

    .. math::
        t3 = c_1e_6 + c_2e_5 + c_3e_4 + c_4e_3

        e_1 = ema(data)

        e_2 = ema(e_1)

        e_3 = ema(e_2)

        e_4 = ema(e_3)

        e_5 = ema(e_4)

        e_6 = ema(e_5)

        c_1 = -a^3

        c_2 = 3a^2 + 3a^3

        c_3 = –6a^2–3a–3a^3

        c_4 = 1 + 3a + a^3 + 3a^2

    where *a* is the volume factor and is typically set to `0.7`.
    """
    if volume_factor < 0:
        raise ValueError("Param 'volume_factor' needs to be bigger than zero")

    return t3_calc(data, periods, volume_factor)


def tma(data, periods):
    """
    .. Triangular Moving Average

    Parameters
    ----------
    data : `ndarray`
        An array containing values.
    periods : `int`
        Number of periods to be used.

    Returns
    -------
    `ndarray`
        An array containing calculated triangular moving average values.

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ... 
    >>> # Load a sample dataframe.
    >>> df = ql.load_sample('MSFT')
    >>> print(df['close'].dtype)
    float64
    >>> tma = ql.tma(df['close'], periods = 10)
    >>> print(tma)
    [nan nan nan ... 208.93833333 209.115 209.684]

    Notes
    -----
    The triangular moving average is calculated by taking an sma of an sma.

    .. math::
        tma = sma(sma(price, n_1), n_2)

    As seen above, two different periods are used in this
    implementation. If the parameter `periods` is even

    .. math::
        n_1 = \\frac{periods}{2}

        n_2 = \\frac{periods}{2} + 1

    otherwise they are rounded up after the following calculation

    .. math::
        n_1 = n_2 = \\frac{periods + 1}{2}
    """
    return tma_calc(data, periods)

def smma(data, periods):
    """
    .. Smoothed Moving Average

    Parameters
    ----------
    data : `ndarray`
        An array containing values.
    periods : `int`
        Number of periods to be used.

    Returns
    -------
    `ndarray`
        An array containing calculated smoothed moving average values.

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ... 
    >>> # Load a sample dataframe.
    >>> df = ql.load_sample('MSFT')
    >>> print(df['close'].dtype)
    float64
    >>> smma = ql.smma(df['close'], periods = 10)
    >>> print(smma)
    [nan nan nan ... 208.85810754 209.43029679 209.78926711]
    """
    return smma_calc(data, periods)

def lwma(data, periods):
    """
    .. Linear Weighted Moving Average

    Parameters
    ----------
    data : `ndarray`
        An array containing values.
    periods : `int`
        Number of periods to be used.

    Returns
    -------
    `ndarray`
        An array containing calculated linear weighted moving average values.

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ... 
    >>> # Load a sample dataframe.
    >>> df = ql.load_sample('MSFT')
    >>> print(df['close'].dtype)
    float64
    >>> lwma = ql.lwma(df['close'], periods = 10)
    >>> print(lwma)
    [nan nan nan ... 209.50327273 210.35927273 210.96381818]

    Notes
    -----
    The linear weighted moving average is calculated similar to the simple
    moving average, except that the values aren't equally weighted, but
    linearly weighted with the highest weight going first and then decreasing
    linearly. This implementation use the number of periods as the highest
    weight, and thereafter decreasing down to one.

    .. math:: lwma_K = \\frac{price_K \cdot w_n + price_{K-1} \cdot w_{n-1} \\
            + ... + price_{K-(n-1)} \cdot w_1}{\sum_{i=1}^n w_i},

    where :math:`w_1 = 1, w_2 = 2,... w_n = n`
    """
    return lwma_calc(data, periods)

def wc(high, low, close):
    """
    .. Weighted Close

    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    close : `ndarray`
        An array containing closing prices.

    Returns
    -------
    `ndarray`
        An array containing calculated weighted close values.

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ... 
    >>> # Load a sample dataframe.
    >>> df = ql.load_sample('MSFT')
    >>> wc = ql.wc(df['high'], df['low'], df['close'])
    >>> print(wc)
    [153.1975 154.09 154.3075 ... 210.1875 213.2675 213.785]

    Notes
    -----
    The weighted close is defined as:

    .. math:: wc_K = \\frac{2 \cdot close_K + high_K + low_K}{4}

    """
    return wc_calc(close, high, low)






"""
@ Qufilab, 2020.
@ Anton Normelius

Python interface for momentum indicators.

"""
import time
import numpy as np 

from qufilab.indicators._momentum import *

def rsi(price, period, rsi_type = "smoothed"):
    """
    .. Relative strength index
    
    Parameters
    ----------
    price : `ndarray`
        Array of type float64 or float32 containing the data to calculate rsi from.
    period : `int`
        Number of periods to be used.
    rsi_type : {'smoothed', 'standard'}, optional
        Specify what kind of averaging should be used for calculating the average gain/
        average loss. Standard is the Wilder's smoothing.
    
    Returns
    -------
    `ndarray`
        Returns a numpy ndarray with type float64 or float32.
    """
    return rsi_calc(price, period, rsi_type.lower())

def macd(price):
    """
    .. MACD

    Parameters
    ----------
    price : `ndarray`
        Array of type float64 or float32 containing the prices to calculate macd from.

    Returns
    -------
    macd : `ndarray`
        Array of type float64 or float32 containing the macd values.
    signal : `ndarray`
        Array of type float64 or float32 containing the signal values.
    """
    return macd_calc(price)

def willr(close, high, low, period):
    """
    .. William's R

    Parameters
    ----------
    close : `ndarray`
        Array of type float64 or float32 containing the closing prices.
    high : `ndarray`
        Array of type float64 or float32 containing the high prices.
    low : `ndarray`
        Array of type float64 or float32 containing the low prices.
    period : `int`
        Number of periods to be used.

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the william's r values.
    """
    return willr_calc(close, high, low, period)

def roc(price, period):
    """
    .. Price Rate of Change

    Parameters
    ----------
    price : `ndarray`
        Array of type float64 or float32 containing price values.
    period : `int`
        Number of periods to be used.

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the calculated rate
        of change values.
    """
    return roc_calc(price, period)

def vpt(price, volume):
    """
    .. Volume Price Trend

    Parameters
    ----------
    price : `ndarray`
        Array of type float64 or float32 containing price values.
    volume : `ndarray`
        Array of type float64 or float32 containing volume values.

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the calculated volume
        price trend values.
    """
    return vpt_calc(price, volume)

def mi(price, period):
    """
    .. Momentum Indicator

    Parameters
    ----------
    price : `ndarray`
        Array of type float64 or float32 containing price values.
    period : `int`
        Number of periods to be used.

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the calculated momentum
        indicator values.
    """
    return mi_calc(price, period)

def apo(price, period_slow = 26, period_fast = 12, ma = "sma"):
    """
    .. Absolute Price Oscillator

    Parameters
    ----------
    price : `ndarray`
        Array of type float64 or float32 containing price values.
    period_slow : `int`, optional
        Number of periods for the slow moving average.
        Default to 26.
    period_fast : `int`, optional
        Number of fast periods for the fast moving average.
        Default to 12.
    ma : {'sma', 'ema'}, optional
        Type of moving average to be used. 
        Default to 'sma'

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the calculated absolute
        price oscillator values.
    """
    if ma.lower() not in ["sma", "ema"]:
        raise ValueError("param 'ma' needs to be 'ema' or 'sma'")

    return apo_calc(price, period_slow, period_fast, ma.lower())

def bop(high, low, open_, close):
    """
    .. Balance of Power

    Parameters
    ----------
    high : `ndarray`
        Array of type float64 or float32 containing high prices.
    low : `ndarray`
        Array of type float64 or float32 containing low prices.
    open_ : `ndarray`
        Array of type float64 or float32 containing open prices.
    close : `ndarray`
        Array of type float64 or float32 containing closing prices.

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the calculated 
        balance of power values.
    """
    return bop_calc(high, low, open_, close)

def cmo(close, period):
    """
    .. Chande Momentum Indicator

    Parameters
    ----------
    close : `ndarray`
        Array of type float64 or float32 containing closing prices.
    period : `int`
        Number of periods to be used.

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the calculated chande
        momentum values.
    """
    return cmo_calc(close, period)

def mfi(high, low, close, volume, period):
    """
    .. Money Flow Index

    Parameters
    ----------
    high : `ndarray`
        Array of type float64 or float32 containing high prices.
    low : `ndarray`
        Array of type float64 or float32 containing low prices.
    close : `ndarray`
        Array of type float64 or float32 containing closing prices.
    volume : `ndarray`
        Array of type float64 or float32 containing volume values.
    period : `int`
        Number of periods to be used.

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the calculated money
        flow index values.
    """
    return mfi_calc(high, low, close, volume, period)

def ppo(price, period_fast = 12, period_slow = 26, ma = "ema"):
    """
    .. Percentage Price Oscillator
    
    Parameters
    ----------
    price : `ndarray`
        Array of type float64 or float32 containing price values.
    period_fast : `int`, optional
        Number of fast periods for the fast moving average.
        Default to 12.
    period_slow : `int`, optional
        Number of periods for the slow moving average.
        Default to 26.
    ma : {'ema', 'sma'}, optional
        Type of moving average to be used. 
        Default to 'ema'

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the calculated 
        percentage price values.
    """
    if ma.lower() not in ["sma", "ema"]:
        raise ValueError("Param 'ma' needs to be 'sma' or 'ema'")

    return ppo_calc(price, period_fast, period_slow, ma.lower())

#def stochastic(close, high, low, mode = "fast", period_k = 10, method = "ema"):
#    """
#    Returns %K and %D stochastics.
#
#    :param close: Closing prices.
#    :type close: List.
#    :param high: High prices.
#    :type high: List.
#    :param low: Low prices.
#    :type low: List.
#    :param mode: Specify whether to calculate 'fast' or 'slow' stochastic.
#    :type mode: Str.
#    :param period_k: Specify the number of periods used in the Stochastic (%K)
#        calculation.
#    :type period_k: Int.
#    :param method: Specify the method that is used to calculate Stochastic (%D) 
#        calculation.
#    :type method: Str.
#    :return tuple: A tuple containing (%K, %D).
#    """
#    if method and method.lower() not in ['sma', 'ema']:
#        raise ValueError("Param 'method' needs to be 'sma' or 'ema'.")
#    
#    if mode and mode.lower() not in ['fast', 'slow']:
#        raise ValueError("Param 'mode' needs to be 'fast' or 'slow'.")
#    
#    if not isinstance(period_k, int):
#        raise ValueError("Param 'period_k' needs to be an int.")
#
#    return stochastic_calc(close, high, low, mode, period_k, method)


def cci(close, high, low, period = 20):
    """
    .. Commodity Channel Index

    Parameters
    ----------
    close : `ndarray`
        Array of type float64 or float32 containing closing prices.
    high : `ndarray`
        Array of type float64 or float32 containing high prices.
    low : `ndarray`
        Array of type float64 or float32 containing low prices.
    period : `int`, optional
        Number of periods to be used.
        Defaults to 20.

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the calculated commodity
        channel index values.
    """
    return cci_calc(close, high, low, period)

def aroon(high, low, period = 20):
    """
    .. Aroon Indicator
    
    Parameters
    ----------
    high : `ndarray`
        Array of type float64 or float32 containing high prices.
    low : `ndarray`
        Array of type float64 or float32 containing low prices.
    period : `int`, optional
        Number of periods to be used.
        Defaults to 20.

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the calculated
        aroon indicator values.
    """
    return aroon_calc(high, low, period)

#def tsi(close, period = 25, period_double = 13):
#    return tsi_calc(close, period, period_double)



"""
@ Qufilab, 2020.
@ Anton Normelius

Python interface for volatility indicators.

"""
import numpy as np 

from qufilab.indicators._volatility import *

def bbands(price, period, deviation = 2):
    """
    .. Bollinger Bands

    Parameters
    ----------
    price : `ndarray`
        Array of type float64 or float32 containing the prices.
    period : `int`
        Number of periods to be used.
    deviation : `int`, optional
        Number of standard deviations from the mean.
        Defaults to 20.

    Returns
    -------
    upper : `ndarray`
        Upper bollinger band.
    middle : `ndarray`
        middle bollinger band.
    lower : `ndarray`
        lower bollinger band.
    """
    return bbands_calc(price, period, deviation)

def kc(close, high, low, period = 20, period_atr = 20, deviation = 2):
    """
    .. Keltner channels

    Parameters
    ----------
    close : `ndarray`
        Array of type float64 or float32 containing the closing prices.
    high : `ndarray`
        Array of type float64 or float32 containing the high prices.
    low : `ndarray`
        Array of type float64 or float32 containing the low prices.
    period : `int`, optional
        Number of periods to be used.
        Defaults to 20.
    period_atr : `int`, optional
        Number of periods to be used for the average true range calculations.
        Defaults to 20.
    deviation : `int`, optional
        Number of deviations from the mean.
        Defaults to 2.

    Returns
    -------
    upper : `ndarray`
        Upper keltner band.
    middle : `ndarray`
        middle keltner band.
    lower : `ndarray`
        lower keltner band.
    """
    return kc_calc(close, high, low, period, period_atr, deviation)

def atr(close, high, low, period):
    """
    .. Average true range

    Parameters
    ----------
    close : `ndarray`
        Array of type float64 or float32 containing the closing prices.
    high : `ndarray`
        Array of type float64 or float32 containing the high prices.
    low : `ndarray`
        Array of type float64 or float32 containing the low prices.
    period : `int`
        Number of periods to be used.

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the calculated
        average true range values.
    """
    return atr_calc(close, high, low, period)

def cv(high, low, period = 10, smoothing_period = 10):
    """
    .. Chaikin volatility

    Parameters
    ----------
    high : `ndarray`
        Array of type float64 or float32 containing the high prices.
    low : `ndarray`
        Array of type float64 or float32 containing the low prices.
    period : `int`, optional
        Number of periods to be used.
        Defaults to 10.
    smooting_period : `int`, optional
        Number of periods to be used for smoothing chaikin volatility values.
        Defaults to 10.

    Returns
    -------
    `ndarray`
        Array of type float64 or float32 containing the calculated
        chaikin volatility values.
    """
    return cv_calc(high, low, period, smoothing_period)



"""
@ Qufilab, 2020.
@ Anton Normelius

Python interface for statistics indicators.

"""
import numpy as np 

from qufilab.indicators._stat import *

def std(data, periods, normalize = True):
    """
    .. Standard Deviation

    Parameters
    ----------
    data : `ndarray`
        An array containing values.
    periods : `int`
        Number of periods to be used.
    normalize : `bool`, optional
        Specify whether to normalize the standard deviation with 
        n - 1 instead of n.
        Defaults to True.

    Returns
    -------
    `ndarray`
        An array containing standard deviation values for the specified periods.

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ...
    >>> # Load a sample dataframe.
    >>> df = ql.load_sample('MSFT')
    >>> print(df['close'].dtype)
    float64
    >>> sma = ql.std(df['close'], periods = 10)
    >>> print(sma)
    [nan nan nan ... 3.31897842 2.9632574  3.02394683]
    """
    return std_calc(data, periods, normalize)

def var(data, periods, normalize = True):
    """
    .. Variance

    Parameters
    ----------
    data : `ndarray`
        An array containing values.
    periods : `int`
        Number of periods to be used.
    normalize : `bool`, optional
        Specify whether to normalize the standard deviation with 
        n - 1 instead of n.
        Defaults to `True`.

    Returns
    -------
    `ndarray`
        An array containing variance values.

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ...
    >>> # Load a sample dataframe.
    >>> df = ql.load_sample('MSFT')
    >>> print(df['close'].dtype)
    float64
    >>> var = ql.var(df['close'], periods = 10)
    >>> print(var)
    [nan nan nan ... 11.01561778  8.78089444 9.14425444]
    """
    return var_calc(data, periods, normalize)

def cov(data, market, periods, normalize = True):
    """
    .. Covariance

    Parameters
    ----------
    data : `ndarray`
        An array containing values.
    market : `ndarray`
        An array containing market values to be used as the comparison
        when calculating beta.
    periods : `int`
        Number of periods to be used.
    normalize : `bool`, optional
        Specify whether to normalize covariance with 
        n - 1 instead of n.
        Defaults to `True`.

    Returns
    -------
    `ndarray`
        An array containing covariance values.

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ...
    >>> # Load sample dataframe.
    >>> df = ql.load_sample('MSFT')
    >>> df_market = ql.load_sample('DJI')
    >>> cov = ql.cov(df['close'], df_market['close'], periods = 10)
    >>> print(cov)
    [nan nan nan ... -360.37842558  -99.1077715 60.84627274]
    """
    return cov_calc(data, market, periods, normalize)

def beta(data, market, periods, normalize = False):
    """
    .. Beta

    Parameters
    ----------
    data : `ndarray`
        An array containing values.
    market : `ndarray`
        An array containing market values to be used as the comparison
        when calculating beta.
    periods : `int`
        Number of periods to be used.
    normalize : `bool`, optional
        Specify whether to normalize the standard deviation calculation
        within the beta calculation with n - 1 instead of n.
        Defaults to False.

    Returns
    -------
    `ndarray`
        An array containing beta values.

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ...
    >>> # Load sample dataframe.
    >>> df = ql.load_sample('MSFT')
    >>> df_market = ql.load_sample('DJI')
    >>> beta = ql.beta(df['close'], df_market['close'], periods = 10)
    >>> print(beta)
    [nan nan nan ... 0.67027616 0.45641977 0.3169785]
    """
    return beta_calc(data, market, periods, normalize)

def pct_change(data, periods):
    """
    .. Percentage Change

    Parameters
    ----------
    data : `ndarray`
        An array containing values.
    periods : `int`
        Number of periods to be used.

    Returns
    -------
    `ndarray`
        An array containing percentage change for the specified periods.

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ...
    >>> # Load sample dataframe.
    >>> df = ql.load_sample('MSFT')
    >>> pct_change = ql.pct_change(df['close'], periods = 10)
    >>> print(pct_change)
    [nan nan nan ... -1.52155537 -0.81811879 0.25414157]
    """
    return pct_change_calc(data, periods)



"""
@ Quant
@ Anton Normelius, 2020.

Interface for the quantitative models.
"""

import pandas as pd

#from quant.models import openChange

class Models():
    def __init__(self, data):
        """
        Params:
            data (pd DataFrame).
        """
        self.data = data
        self.close = self.data['close'].tolist()
        self.open = self.data['open'].tolist()
        self.high = self.data['high'].tolist()
        self.low = self.data['low'].tolist()
        self.date = self.data['date'].tolist()
    
    def openChange(self, mode, entry, direction, cover, stop_loss = 0.0, 
            show_positions = False):
        """
        Model to analyse change on initial opening.
        """
        if entry == 0:
            raise ValueError("Param entry can't be 0")

        mode = mode.lower()
        if mode not in ['daily', 'percentage']:
            raise ValueError("Param mode needs to be 'daily' or 'percentage'.")

        if mode == 'daily':
            if not isinstance(cover, int):
                raise ValueError("Param cover needs to be an int," \
                        " specifying number of days before covering position.")

            openChange(self.date, self.close, self.open, self.high, self.low,
                    entry, mode, direction, cover, stop_loss, show_positions)

        elif mode == 'percentage':
            if (stop_loss == 0.0):
                raise ValueError("Param stop_loss needs to be specified")

            if (cover <= 0 or cover >= 1):
                raise ValueError("Param cover needs to be bigger than zero" \
                        " but less than one.")


            #openChange(self.date, self.close, self.open, self.high, self.low,
            #        entry, mode, direction, cover, stop_loss, show_positions)

    
    #def trendCrossover(self, indicator, periods, deviation = 0):
    #    """
    #    Model to analyze positions when price cross trend indicator.
    #    """
    #    indicators = ['sma', 'ema', 'smma', 'lwma', 'bollinger_bands']

    #    indicator = indicator.lower()

    #    if indicator not in indicators:
    #        raise ValueError("param 'indicator' needs to be specified")
    #    
    #    if not periods:
    #        raise ValueError("param 'periods' needs to be speicified.")
    #
    #    if indicator == 'bb' and std_dev == 0:
    #        raise ValueError("param 'std_dev' needs to be speicified.")

    #    if indicator == 'bb' and not isinstance(std_dev, int):
    #        raise ValueError("param 'std_dev' needs to be an int.")

    #    trendCrossover(self.date, self.close, self.open, self.high, self.low,
    #            indicator, periods, deviation)








"""
@ QufiLab, 2020.
@ Anton Normelius

Python interface for bullish patterns.

"""
import numpy as np

from qufilab.patterns._bullish import *

def hammer(high, low, open_, close, periods = 10, shadow_margin = 5.0):
    """
    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    periods : `int`, optional
        Specifying number of periods for trend identification.
    shadow_margin : `float`, optional
        Specify what margin should be allowed for the shadows. By using i.e.
        5%, upper shadow can be as long as 5% of the candlestick body size. 
        This exist to allow some margin and not exclude the shadows entirely.

    Returns
    -------
    hammer : `ndarray`
        A numpy array of type bool specifying true whether
        a pattern has been found or false otherwise. 

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ... 
    >>> df = ql.load_sample('MSFT')
    >>> hammer = ql.hammer(df['high'], df['low'], df['open'], df['close'])
    >>> print(hammer)
    [False False False ... False False False]
    
    Notes
    -----
    Observe that the lower shadow shall be bigger than 2x the body, but lower
    than 3x the body.

    .. image:: images/hammer.png

    """
    hammer_type = "hammer"
    hammer = hammer_calc(high, low, open_, close, periods, hammer_type, shadow_margin)
    return hammer
    
def inverted_hammer(high, low, open_, close, periods = 10, shadow_margin = 5.0):
    """
    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    periods : `int`, optional
        Specifying number of periods for trend identification.
    shadow_margin : `float`, optional
        Specify what margin should be allowed for the shadows. By using i.e.
        5%, lower shadow can be as long as 5% of the candlestick body size. 
        This exist to allow some margin and not exclude the shadows entirely.

    Returns
    -------
    inverted_hammer : `ndarray`
        A numpy ndarray of type bool specifying true whether
        a pattern has been found or false otherwise. 

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ... 
    >>> df = ql.load_sample('MSFT')
    >>> inverted_hammer = ql.inverted_hammer(df['high'], df['low'], df['open'], df['close'])
    >>> print(inverted_hammer)
    [False False False ... False False False]


    """
    hammer_type = "inverted_hammer"
    hammer = hammer_calc(high, low, open_, close, periods, hammer_type, shadow_margin)
    return hammer

def doji(high, low, open_, close, periods = 10):
    """
    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    periods : `int`, optional
        Specifying number of periods for trend identification.

    Returns
    -------
    doji : `ndarray`
        A numpy ndarray of type bool specifying true whether
        a pattern has been found or false otherwise. 

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ... 
    >>> df = ql.load_sample('MSFT')
    >>> doji = ql.doji(df['high'], df['low'], df['open'], df['close'])
    >>> print(doji)
    [False False False ... False False False]

    """
    doji = doji_calc(high, low, open_, close, periods)
    return doji

def dragonfly_doji(high, low, open_, close, periods = 10):
    """
    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    periods : `int`, optional
        Specifying number of periods for trend identification.

    Returns
    -------
    dragonfly_doji : `ndarray`
        A numpy ndarray of type bool specifying true whether
        a pattern has been found or false otherwise. 

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ... 
    >>> df = ql.load_sample('MSFT')
    >>> dragonfly_doji = ql.dragonfly_doji(df['high'], df['low'], df['open'], df['close'])
    >>> print(dragonfly_doji)
    [False False False ... False False False]

    """
    dragonfly_doji = dragonfly_doji_calc(high, low, open_, close, periods)
    return dragonfly_doji

def marubozu_white(high, low, open_, close, shadow_margin = 5.0, periods = 10):
    """
    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    shadow_margin : `float`, optional
        Specify what margin should be allowed for the shadows. By using for 
        example 5%, both the lower and upper shadow can be as high as 5%
        of the candlestick body size. This exist to allow some margin (not
        restrict to no shadow).
    periods : `int`, optional
        Specifying number of periods for trend identification.

    Returns
    -------
    marubozu_white : `ndarray`
        A numpy ndarray of type bool specifying true whether
        a pattern has been found or false otherwise. 

    Examples
    --------
    >>> import qufilab as ql
    >>> import numpy as np
    ... 
    >>> df = ql.load_sample('MSFT')
    >>> marubozu_white = ql.marubozu_white(df['high'], df['low'], df['open'], df['close'])
    >>> print(marubozu_white)
    [False False False ... False False False]
    """
    marubozu_white = marubozu_white_calc(high, low, open_, close, shadow_margin, periods)
    return marubozu_white

def marubozu_black(high, low, open_, close, shadow_margin = 5.0, periods = 10):
    """
    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    shadow_margin : `float`, optional
        Specify what margin should be allowed for the shadows. By using for 
        example 5%, both the lower and upper shadow can be as high as 5%
        of the candlestick body size. This exist to allow some margin (not
        restrict to no shadow).
    periods : `int`, optional
        Specifying number of periods for trend identification.

    Returns
    -------
    marubozu_black : `ndarray`
        A numpy ndarray of type bool specifying true whether
        a pattern has been found or false otherwise. 
    """
    pattern = marubozu_black_calc(high, low, open_, close, shadow_margin, periods)
    return pattern

def spinning_top_white(high, low, open_, close, periods = 10):
    """
    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    periods : `int`, optional
        Specifying number of periods for trend identification.

    Returns
    -------
    spinning_top_white : `ndarray`
        A numpy ndarray of type bool specifying true whether
        a pattern has been found or false otherwise. 
    """
    spinning_top_white = spinning_top_white_calc(high, low, open_, close, periods)
    return spinning_top_white

def engulfing_bull(high, low, open_, close, periods = 10):
    """
    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    periods : `int`, optional
        Specifying number of periods for trend identification.

    Returns
    -------
    engulfing : `ndarray`
        A numpy ndarray of type bool specifying true whether
        a pattern has been found or false otherwise. 

    """
    engulfing_type = "bull"
    engulfing = engulfing_calc(high, low, open_, close, periods, engulfing_type)
    return engulfing

def engulfing_bear(high, low, open_, close, periods = 10):
    """
    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    periods : `int`, optional
        Specifying number of periods for trend identification.

    Returns
    -------
    engulfing : `ndarray`
        A numpy ndarray of type bool specifying true whether
        a pattern has been found or false otherwise. 
    """
    engulfing_type = "bear"
    engulfing = engulfing_calc(high, low, open_, close, periods, engulfing_type)
    return engulfing

def harami_bull(high, low, open_, close, periods = 10):
    """
    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    periods : `int`, optional
        Specifying number of periods for trend identification.

    Returns
    -------
    harami_bull : `ndarray`
        A numpy ndarray of type bool specifying true whether
        a pattern has been found or false otherwise. 
    """
    harami_type = "bull"
    harami = harami_calc(high, low, open_, close, periods, harami_type)
    return harami

def harami_bear(high, low, open_, close, periods = 10):
    """
    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    periods : `int`, optional
        Specifying number of periods for trend identification.

    Returns
    -------
    harami_bear : `ndarray`
        A numpy ndarray of type bool specifying true whether
        a pattern has been found or false otherwise. 
    """
    harami_type = "bear"
    harami = harami_calc(high, low, open_, close, periods, harami_type)
    return harami

def kicking_bull(high, low, open_, close, periods = 10, shadow_margin = 5.0):
    """
    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    periods : `int`, optional
        Specifying number of periods for trend identification.
    shadow_margin : `float`, optional
        Specify what margin should be allowed for the shadows. By using for 
        example 5%, both the lower and upper shadow can be as high as 5%
        of the candlestick body size. This exist to allow some margin (not
        restrict to no shadow).

    Returns
    -------
    kicking_bull : `ndarray`
        A numpy ndarray of type bool specifying true whether
        a pattern has been found or false otherwise. 
    """
    kicking_type = "bull"
    kicking = kicking_calc(high, low, open_, close, periods, kicking_type, shadow_margin)
    return kicking
    
def kicking_bear(high, low, open_, close, periods = 10, shadow_margin = 5.0):
    """
    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    periods : `int`, optional
        Specifying number of periods for trend identification.
    shadow_margin : `float`, optional
        Specify what margin should be allowed for the shadows. By using for 
        example 5%, both the lower and upper shadow can be as high as 5%
        of the candlestick body size. This exist to allow some margin (not
        restrict to no shadow).

    Returns
    -------
    kicking_bear : `ndarray`
        A numpy ndarray of type bool specifying true whether
        a pattern has been found or false otherwise. 
    """
    kicking_type = "bear"
    kicking = kicking_calc(high, low, open_, close, periods, kicking_type, shadow_margin)
    return kicking

def piercing(high, low, open_, close, periods = 10):
    """
    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    periods : `int`, optional
        Specifying number of periods for trend identification.

    Returns
    -------
    piercing : `ndarray`
        A numpy ndarray of type bool specifying true whether
        a pattern has been found or false otherwise. 
    """
    piercing = piercing_calc(high, low, open_, close, periods)
    return piercing

def tws(high, low, open_, close, periods = 10):
    """
    Three White Soldiers

    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    periods : `int`, optional
        Specifying number of periods for trend identification.

    Returns
    -------
    tws : `ndarray`
        A numpy ndarray of type bool specifying true whether
        a pattern has been found or false otherwise. 
    """
    tws = tws_calc(high, low, open_, close, periods)
    return tws

def abandoned_baby_bull(high, low, open_, close, periods = 10):
    """
    Abandoned Baby Bull

    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    periods : `int`, optional
        Specifying number of periods for trend identification.

    Returns
    -------
    pattern : `ndarray`
        A numpy ndarray of type bool specifying true whether
        a pattern has been found or false otherwise. 
    """
    type_ = "bull"
    pattern = abandoned_baby_calc(high, low, open_, close, periods, type_)
    return pattern

def abandoned_baby_bear(high, low, open_, close, periods = 10):
    """
    Abandoned Baby Bear

    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    periods : `int`, optional
        Specifying number of periods for trend identification.

    Returns
    -------
    pattern : `ndarray`
        A numpy ndarray of type bool specifying true whether
        a pattern has been found or false otherwise. 
    """
    type_ = "bear"
    pattern = abandoned_baby_calc(high, low, open_, close, periods, type_)
    return pattern


def belthold_bull(high, low, open_, close, periods = 10, shadow_margin = 5.0):
    """
    Belt Hold Bull

    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    periods : `int`, optional
        Specifying number of periods for trend identification.
    shadow_margin : `float`, optional
        Specify what margin should be allowed for the shadows. By using for 
        example 5%, both the lower and upper shadow can be as high as 5%
        of the candlestick body size. This exist to allow some margin (not
        restrict to no shadow).

    Returns
    -------
    pattern : `ndarray`
        A numpy ndarray of type bool specifying true whether
        a pattern has been found or false otherwise. 
    """
    type_ = "bull"
    pattern = belthold_calc(high, low, open_, close, periods, type_, shadow_margin)
    return pattern
    

def belthold_bear(high, low, open_, close, periods = 10, shadow_margin = 5.0):
    """
    Belt Hold Bear

    Parameters
    ----------
    high : `ndarray`
        An array containing high prices.
    low : `ndarray`
        An array containing low prices.
    open_ : `ndarray`
        An array containing open prices.
    close : `ndarray`
        An array containing close prices.
    periods : `int`, optional
        Specifying number of periods for trend identification.
    shadow_margin : `float`, optional
        Specify what margin should be allowed for the shadows. By using for 
        example 5%, both the lower and upper shadow can be as high as 5%
        of the candlestick body size. This exist to allow some margin (not
        restrict to no shadow).

    Returns
    -------
    pattern : `ndarray`
        A numpy ndarray of type bool specifying true whether
        a pattern has been found or false otherwise. 
    """
    type_ = "bear"
    pattern = belthold_calc(high, low, open_, close, periods, type_, shadow_margin)
    return pattern



from distutils.util import convert_path
import os
import re
import sys
import platform
import subprocess

from setuptools import setup, Extension, find_packages
from setuptools.command.build_ext import build_ext
from distutils.version import LooseVersion

NAME = "qufilab"
AUTHOR = "Anton Normelius"
EMAIL = "a.normelius@gmail.com"
URL = "https://github.com/normelius/qufilab"


# Read readme.
with open("README.md", "r") as fh:
    long_description = fh.read()

# Read requirements.
with open('requirements.txt') as f:
    required = f.read().splitlines()

class get_pybind_include(object):
    """
    Helper class to determine the pybind11 include path
    The purpose of this class is to postpone importing pybind11
    until it is actually installed, so that the ``get_include()``
    method can be invoked.
    """
    def __str__(self):
        import pybind11
        return pybind11.get_include()

def has_flag(compiler, flagname):
    """
    Return a boolean indicating whether a flag name is supported on
    the specified compiler.
    """
    import tempfile
    import os
    with tempfile.NamedTemporaryFile('w', suffix='.cc', delete=False) as f:
        f.write('int main (int argc, char **argv) { return 0; }')
        fname = f.name
    try:
        compiler.compile([fname], extra_postargs=[flagname])
    except setuptools.distutils.errors.CompileError:
        return False
    finally:
        try:
            os.remove(fname)
        except OSError:
            pass
    return True


def cpp_flag(compiler):
    """Return the -std=c++[11/14/17] compiler flag.
    The newer version is prefered over c++11 (when it is available).
    """
    #flags = ['-std=c++17', '-std=c++14', '-std=c++11']
    flags = ['-std=c++11']

    for flag in flags:
        if has_flag(compiler, flag):
            return flag

    raise RuntimeError('Unsupported compiler -- at least C++11 support '
                       'is needed!')


class BuildExt(build_ext):
    """A custom build extension for adding compiler-specific options."""
    c_opts = {
        'msvc': ['/EHsc'],
        'unix': [],
    }
    l_opts = {
        'msvc': [],
        'unix': [],
    }

    if sys.platform == 'darwin':
        darwin_opts = ['-stdlib=libc++', '-mmacosx-version-min=10.7']
        c_opts['unix'] += darwin_opts
        l_opts['unix'] += darwin_opts

    def build_extensions(self):
        ct = self.compiler.compiler_type
        opts = self.c_opts.get(ct, [])
        link_opts = self.l_opts.get(ct, [])
        if ct == 'unix':
            opts.append(cpp_flag(self.compiler))
            if has_flag(self.compiler, '-fvisibility=hidden'):
                opts.append('-fvisibility=hidden')

        for ext in self.extensions:
            ext.define_macros = [('VERSION_INFO', '"{}"'.format(self.distribution.get_version()))]
            ext.extra_compile_args = opts
            ext.extra_link_args = link_opts
        build_ext.build_extensions(self)

"""
Each individual extension, i.e. .so file that needs to be compiled and created
in order for the qufilab package to successfully import these files needs to be
included below.
Each extension needs to have it's included source files, i.e. .cc files (check
implementation and see which files needs to be included).

OBSERVE! If the source-files haven't been included correctly, python will
raise an ImportError. For example, if 'stat' extension doesn't include 
'trend.cc' sourcefile, the function 'std (standard deviation)' won't work since it
depends on functions from the trend source file.
"""
ext_modules = [
    # Models extension
    #Extension(
    #    'qufilab.models',
    #    sorted(['qufilab/models.cc',
    #        'qufilab/common/time.cc', 
    #        'qufilab/indicators/_volatility.cc',
    #        'qufilab/indicators/_trend.cc']),
    #    include_dirs=[
    #        get_pybind_include(),
    #    ],
    #    language='c++'
    #),
    # Trend extension
    Extension(
        'qufilab.indicators._trend',
        sorted(['qufilab/indicators/_trend.cc']),
        include_dirs=[
            get_pybind_include(),
        ],
        language='c++'
    ),
    # Volatility extension
    Extension(
        'qufilab.indicators._volatility',
        sorted(['qufilab/indicators/_volatility.cc',
            'qufilab/indicators/_trend.cc',
            'qufilab/indicators/_stat.cc']),
        include_dirs=[
            get_pybind_include(),
        ],
        language='c++'
    ),
    ## Momentum extension
    Extension(
        'qufilab.indicators._momentum',
        sorted(['qufilab/indicators/_momentum.cc',
            'qufilab/indicators/_trend.cc']),
        include_dirs=[
            get_pybind_include(),
        ],
        language='c++'
    ),
    ## Volume extension
    Extension(
        'qufilab.indicators._volume',
        sorted(['qufilab/indicators/_volume.cc',
            'qufilab/indicators/_trend.cc']),
        include_dirs=[
            get_pybind_include(),
        ],
        language='c++'
    ),
    ## Stat extension
    Extension(
        'qufilab.indicators._stat',
        sorted(['qufilab/indicators/_stat.cc',
            'qufilab/indicators/_trend.cc']),
        include_dirs=[
            get_pybind_include(),
        ],
        language='c++'
    ),
    # Patterns
    ## Bullish extension
    Extension(
        'qufilab.patterns._bullish',
        sorted(['qufilab/patterns/_bullish.cc',
            'qufilab/patterns/candlestick.cc',
            'qufilab/indicators/_trend.cc']),
        include_dirs=[
            get_pybind_include(),
        ],
        language='c++'
    ),
]

PACKAGES = ['qufilab']

setup(
    name=NAME,
    author=AUTHOR,
    author_email=EMAIL,
    url=URL,
    packages=find_packages(),
    install_requires=required,
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    ext_modules=ext_modules,
    cmdclass={'build_ext': BuildExt},
    zip_safe = False,
    include_package_data=True
)
    





import pandas as pd
import os

def load_sample(ticker):
    """
    Parameters
    ----------
    ticker : {'AAPL', 'NFLX', 'MSFT'}
        Ticker to load data for.
    
    Returns
    -------
    `DataFrame`
        DataFrame containing `date`, `high`, `low`, `open`, `close`, `volume`.
    """
    if not isinstance(ticker, str):
        raise TypeError("Param 'ticker' needs to be a str.")
    
    if ticker not in ['MSFT', 'NFLX', 'TSLA', 'DJI']:
        raise ValueError("Param 'ticker' needs to be 'MSFT', 'NFLX', 'DJI'" \
                " or 'TSLA'.")

    df = pd.read_csv(os.path.join(os.path.dirname(__file__), ticker + ".csv"))
    pd.to_datetime(df['date'], format = '%Y-%m-%d') 
    return df




#from .main import Models
#from .indicators.interface import *

# Indicators
from .indicators.trend import *
from .indicators.stat import *
from .indicators.volume import *
from .indicators.volatility import *
from .indicators.momentum import *

# Patterns
from .patterns.bullish import *

# Sample data
from .sample.load_sample import *



